;----------------------------------------------------------------------------------
; Spatial Audio
; Coded by Hans Mikelson October 1998
; This spatial audio system included the following features:
;
;  1. Moving sound source with table supplied moving X and Y coordinates
;  2. Volume decreases as 1/r^2
;  3. Filtering of sound due to air absorption
;  4. Doppler effect due to moving sound
;  5. Simplified HRTF for head shadow filtering
;  6. Inter ear time delay
;  7. Speaker cross-talk cancellation based on both inter ear time delay & head shadowing
;----------------------------------------------------------------------------------

sr=44100
kr=22050
ksmps=2
nchnls=2
zakinit 30,30

;----------------------------------------------------------------------------------
; Disk Input Mono
;----------------------------------------------------------------------------------
        instr  4

iamp    =      p4
irate   =      p5
isndin  =      p6
ioutch1 =      p7

ain1    diskin isndin, irate

        zawm   ain1, ioutch1

        endin

;----------------------------------------------------------------------------------
; Disk Input Stereo
;----------------------------------------------------------------------------------
        instr  5

iamp    =      p4
irate   =      p5
isndin  =      p6
ioutch1 =      p7
ioutch2 =      p8

ain1, ain2 diskin isndin, irate

        zawm   ain1, ioutch1
        zawm   ain2, ioutch2
        outs   ain1*iamp, ain2*iamp

        endin

;----------------------------------------------------------------------------------
; Spatial Audio
; Coded by Hans Mikelson October 1998
; This spatial audio system included the following features:
;
; 1. Moving sound source with table supplied moving X and Y coordinates
; 2. Volume decreases as 1/r^2
; 3. Filtering of sound due to air absorption
; 4. Doppler effect due to moving sound
; 5. Simplified HRTF for head shadow filtering
; 6. Inter ear time delay
; 7. Speaker cross-talk cancellation based on both inter ear time delay & head shadowing
;----------------------------------------------------------------------------------
        instr     11

idur    =         p3            ; Duration
iamp    =         p4            ; Amplitude
iexl    =         p5            ; Fixed left ear location X coordinate
iexr    =         iexl+.23      ; Fixed right ear location X coordinate
iey     =         p6            ; Fixed ear location Y coordinate (facing +Y axis)
isxtab  =         p7            ; Moving source table for X coordinate in m
isytab  =         p8            ; Moving source table for Y coordinate in m
iinch   =         p9            ; Audio input channel
ihrtabl =         p10           ; Head shadowing filter for left ear
ihrtabr =         p11           ; Head shadowing filter for right ear
imax    =         500           ; Maximum delay time in ms
ipi     =         3.14159265    ; Pi
ics     =         .333          ; Approx. speed of sound in m/ms

kamp    linseg    0, .002, iamp, idur-.004, iamp, .002, 0 ; Declick Enevelope

ksx     oscil     1, 1/idur, isxtab                       ; Moving Source X
ksy     oscil     1, 1/idur, isytab                       ; Moving Source Y

ksmexl  =         ksx-iexl                                ; Delta X left
ksmexr  =         ksx-iexr                                ; Delta X right
ksmey   =         ksy-iey                                 ; Delta Y
kdistl  =         sqrt(ksmexl*ksmexl+ksmey*ksmey)         ; Distance from left ear to source
kdistr  =         sqrt(ksmexr*ksmexr+ksmey*ksmey)         ; Distance from right ear to source
kdistlnz=         (kdistl==0 ? 1 : kdistl)                ; Avoid divide by zero problem
kdistrnz=         (kdistr==0 ? 1 : kdistr)                ; Avoid divide by zero problem

kangl   =         cosinv(ksmexl/kdistlnz)                   ; Get the magnitude of the angle (0 to Pi)
kangr   =         cosinv(ksmexr/kdistrnz)                   ; Get the magnitude of the angle (0 to Pi)
ksmeynz =         (ksmey==0 ? 1 : ksmey)                  ; Make sure it is not zero
ksign   =         ksmeynz/abs(ksmeynz)/2/ipi              ; Get the sign
kanglel =         kangl*ksign                             ; Get the true angle -Pi to Pi
kangler =         kangr*ksign                             ; Get the true angle -Pi to Pi
kdd4l   =         kdistl/4                                ; Factor for distance and air absorption
kdd4r   =         kdistr/4                                ; Factor for distance and air absorption

ain     zar       iinch                                   ; Input audio

adoppl  vdelay    ain/(1+kdistl*kdistl), kdistl/ics, imax ; Volume decreases as distance^2.
adoppr  vdelay    ain/(1+kdistr*kdistr), kdistr/ics, imax ; Volume decreases as distance^2.
                                                          ; arrival time is delayed by speed of sound, doppler effect 
afltail butterlp  adoppl, 20000/(1+kdd4l)                 ; Air absorption of distant sound
afltair butterlp  adoppr, 20000/(1+kdd4r)                 ; Air absorption of distant sound

khrtfl  tablei    kanglel, ihrtabl, 1, .5                 ; Get the "poor man's" HRTF for left ear
khrtfr  tablei    kangler, ihrtabr, 1, .5                 ; Get the "poor man's" HRTF for right ear

ahrtfl  butterlp  afltail, khrtfl                         ; Apply the filter for head shadowing left ear
ahrtfr  butterlp  afltair, khrtfr                         ; Apply the filter for head shadowing right ear

asctcl  butterlp  ahrtfl, 8000                            ; Simplified head shadowing for speaker cross-talk
asctcr  butterlp  ahrtfr, 8000                            ; Simplified head shadowing for speaker cross-talk

asctcdl delay     asctcl, .0003                           ; Speaker cross-talk cancellation delay left
asctcdr delay     asctcr, .0003                           ; Speaker cross-talk cancellation delay right

        outs      (ahrtfl-asctcdr*.2)*kamp, (ahrtfr-asctcdl*.2)*kamp ; Cancel speaker cross-talk and ouput
        
        zacl      iinch, iinch                            ; Clean up the zak channel

        endin
